@@ -26,6 +26,7 @@ from __future__ import division, print_function, absolute_import
 import os
 import warnings
 import threading
+import contextlib
 from enum import Enum, IntEnum, unique
 
 import attr
@@ -153,6 +154,11 @@ class settings(
                 kwargs['database'] = ExampleDatabase(kwargs['database_file'])
         if not kwargs.get('perform_health_check', True):
             kwargs['suppress_health_check'] = HealthCheck.all()
+        if kwargs.get('max_shrinks') == 0:
+            kwargs['phases'] = tuple(
+                p for p in _validate_phases(kwargs.get('phases'))
+                if p != Phase.shrink
+            )
         self._construction_complete = False
         deprecations = []
         defaults = parent or settings.default
@@ -245,7 +251,7 @@ class settings(
         return test_to_use
 
     @classmethod
-    def define_setting(
+    def _define_setting(
         cls, name, description, default, options=None,
         validator=None, show_default=True, future_default=not_set,
         deprecation_message=None, hide_repr=not_set,
@@ -311,11 +317,25 @@ class settings(
         bits = []
         for name, setting in all_settings.items():
             value = getattr(self, name)
+            # The only settings that are not shown are those that are
+            # deprecated and left at their default values.
             if value != setting.default or not setting.hide_repr:
                 bits.append('%s=%r' % (name, value))
         return 'settings(%s)' % ', '.join(sorted(bits))
 
+    def show_changed(self):
+        bits = []
+        for name, setting in all_settings.items():
+            value = getattr(self, name)
+            if value != setting.default:
+                bits.append('%s=%r' % (name, value))
+        return ', '.join(sorted(bits, key=len))
+
     def __enter__(self):
+        note_deprecation(
+            'Settings should be determined only by global state or with the '
+            '@settings decorator.'
+        )
         default_context_manager = default_variable.with_value(self)
         self.defaults_stack().append(default_context_manager)
         default_context_manager.__enter__()
@@ -380,6 +400,13 @@ class settings(
         settings._assign_default_internal(settings.get_profile(name))
 
 
+@contextlib.contextmanager
+def local_settings(s):
+    default_context_manager = default_variable.with_value(s)
+    with default_context_manager:
+        yield s
+
+
 @attr.s()
 class Setting(object):
     name = attr.ib()
@@ -392,7 +419,7 @@ class Setting(object):
     hide_repr = attr.ib()
 
 
-settings.define_setting(
+settings._define_setting(
     'min_satisfying_examples',
     default=not_set,
     description="""
@@ -405,7 +432,7 @@ max_examples setting.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'max_examples',
     default=100,
     description="""
@@ -414,7 +441,7 @@ counter-example, falsification will terminate.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'max_iterations',
     default=not_set,
     description="""
@@ -426,7 +453,7 @@ useful for this purpose than a user setting.  It no longer has any effect.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'buffer_size',
     default=8 * 1024,
     description="""
@@ -437,13 +464,16 @@ your tests slower.
 )
 
 
-settings.define_setting(
+settings._define_setting(
     'max_shrinks',
-    default=500,
+    default=not_set,
     description="""
-Once this many successful shrinks have been performed, Hypothesis will assume
-something has gone a bit wrong and give up rather than continuing to try to
-shrink the example.
+Passing ``max_shrinks=0`` disables the shrinking phase (see the ``phases``
+setting), but any other value has no effect and uses a general heuristic.
+""",
+    deprecation_message="""
+The max_shrinks setting has been disabled, as internal heuristics are more
+useful for this purpose than a user setting.
 """
 )
 
@@ -455,7 +485,7 @@ def _validate_timeout(n):
         return n
 
 
-settings.define_setting(
+settings._define_setting(
     'timeout',
     default=60,
     description="""
@@ -476,7 +506,7 @@ setting has gone away).
     validator=_validate_timeout
 )
 
-settings.define_setting(
+settings._define_setting(
     'derandomize',
     default=False,
     description="""
@@ -490,7 +520,7 @@ find novel breakages.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'strict',
     default=os.getenv('HYPOTHESIS_STRICT_MODE') == 'true',
     description="""
@@ -521,7 +551,7 @@ def _validate_database(db, _from_db_file=False):
     )
 
 
-settings.define_setting(
+settings._define_setting(
     'database',
     default=not_set,
     show_default=False,
@@ -534,7 +564,7 @@ database, or any path for a directory-based example database.
     validator=_validate_database,
 )
 
-settings.define_setting(
+settings._define_setting(
     'database_file',
     default=not_set,
     show_default=False,
@@ -649,7 +679,7 @@ class Verbosity(IntEnum):
         return 'Verbosity.%s' % (self.name,)
 
 
-settings.define_setting(
+settings._define_setting(
     'verbosity',
     options=tuple(Verbosity),
     default=Verbosity._get_default(),
@@ -667,7 +697,7 @@ def _validate_phases(phases):
     return phases
 
 
-settings.define_setting(
+settings._define_setting(
     'phases',
     default=tuple(Phase),
     description=(
@@ -677,7 +707,7 @@ settings.define_setting(
     validator=_validate_phases,
 )
 
-settings.define_setting(
+settings._define_setting(
     name='stateful_step_count',
     default=50,
     description="""
@@ -685,7 +715,7 @@ Number of steps to run a stateful program for before giving up on it breaking.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'perform_health_check',
     default=not_set,
     description=u"""
@@ -720,14 +750,14 @@ def validate_health_check_suppressions(suppressions):
     return suppressions
 
 
-settings.define_setting(
+settings._define_setting(
     'suppress_health_check',
     default=(),
     description="""A list of health checks to disable.""",
     validator=validate_health_check_suppressions
 )
 
-settings.define_setting(
+settings._define_setting(
     'deadline',
     default=not_set,
     description=u"""
@@ -746,17 +776,15 @@ deadline and have not explicitly set a deadline yourself.
 """
 )
 
-settings.define_setting(
+settings._define_setting(
     'use_coverage',
-    default=True,
+    default=not_set,
+    deprecation_message="""
+use_coverage no longer does anything and can be removed from your settings.
+""",
     description="""
-Whether to use coverage information to improve Hypothesis's ability to find
-bugs.
-
-You should generally leave this turned on unless your code performs
-poorly when run under coverage. If you turn it off, please file a bug report
-or add a comment to an existing one about the problem that prompted you to do
-so.
+A flag to enable a feature that no longer exists. This setting is present
+only for backwards compatibility purposes.
 """
 )
 
@@ -772,17 +800,18 @@ class PrintSettings(Enum):
     """Make an educated guess as to whether it would be appropriate to print
     the blob.
 
-    The current rules are that this will print if both:
+    The current rules are that this will print if:
 
     1. The output from Hypothesis appears to be unsuitable for use with
-       :func:`~hypothesis.example`.
-    2. The output is not too long."""
+       :func:`~hypothesis.example`, and
+    2. The output is not too long, and
+    3. Verbosity is at least normal."""
 
     ALWAYS = 2
     """Always print a blob on failure."""
 
 
-settings.define_setting(
+settings._define_setting(
     'print_blob',
     default=PrintSettings.INFER,
     description="""
